@@ -11,12 +11,11 @@ import functools
 import operator
 import typing
 import warnings
+from packaging.version import Version
 
 import marshmallow
 from marshmallow.orderedset import OrderedSet
 
-from apispec.utils import OpenAPIVersion
-
 
 RegexType = type(re.compile(""))
 
@@ -90,7 +89,7 @@ class FieldConverterMixin:
     """Adds methods for converting marshmallow fields to an OpenAPI properties."""
 
     field_mapping = DEFAULT_FIELD_MAPPING
-    openapi_version: OpenAPIVersion
+    openapi_version: Version
 
     def init_attribute_functions(self):
         self.attribute_functions = [
@@ -106,6 +105,7 @@ class FieldConverterMixin:
             self.field2length,
             self.field2pattern,
             self.metadata2properties,
+            self.enum2properties,
             self.nested2properties,
             self.pluck2properties,
             self.list2properties,
@@ -113,8 +113,10 @@ class FieldConverterMixin:
             self.timedelta2properties,
         ]
 
-    def map_to_openapi_type(self, *args):
-        """Decorator to set mapping for custom fields.
+    def map_to_openapi_type(self, field_cls, *args):
+        """Set mapping for custom field class.
+
+        :param type field_cls: Field class to set mapping for.
 
         ``*args`` can be:
 
@@ -128,11 +130,7 @@ class FieldConverterMixin:
         else:
             raise TypeError("Pass core marshmallow field type or (type, fmt) pair.")
 
-        def inner(field_type):
-            self.field_mapping[field_type] = openapi_type_field
-            return field_type
-
-        return inner
+        self.field_mapping[field_cls] = openapi_type_field
 
     def add_attribute_function(self, func):
         """Method to add an attribute function to the list of attribute functions
@@ -226,7 +224,7 @@ class FieldConverterMixin:
         else:
             default = field.load_default
             if default is not marshmallow.missing and not callable(default):
-                default = field._serialize(default, None, None)  # type:ignore
+                default = field._serialize(default, None, None)
                 ret["default"] = default
         return ret
 
@@ -502,6 +500,22 @@ class FieldConverterMixin:
             ret["x-unit"] = field.precision
         return ret
 
+    def enum2properties(self, field, **kwargs: typing.Any) -> dict:
+        """Return a dictionary of properties from :class:`Enum <marshmallow.fields.Enum` fields.
+
+        :param Field field: A marshmallow field.
+        :rtype: dict
+        """
+        ret = {}
+        if isinstance(field, marshmallow.fields.Enum):
+            ret = self.field2property(field.field)
+            if field.by_value is False:
+                choices = (m for m in field.enum.__members__)
+            else:
+                choices = (m.value for m in field.enum)
+            ret["enum"] = [field.field._serialize(v, None, None) for v in choices]
+        return ret
+
 
 def make_type_list(types):
     """Return a list of types from a type attribute
