@@ -6,12 +6,15 @@ marshmallow :class:`Schemas <marshmallow.Schema>` and :class:`Fields <marshmallo
     This module is treated as private API.
     Users should not need to use this module directly.
 """
-from collections import OrderedDict
+
+from __future__ import annotations
+import typing
+from packaging.version import Version
 
 import marshmallow
 from marshmallow.utils import is_collection
 
-from apispec.utils import OpenAPIVersion
+from apispec import APISpec
 from apispec.exceptions import APISpecError
 from .field_converter import FieldConverterMixin
 from .common import (
@@ -37,7 +40,7 @@ __location_map__ = {
 class OpenAPIConverter(FieldConverterMixin):
     """Adds methods for generating OpenAPI specification from marshmallow schemas and fields.
 
-    :param str|OpenAPIVersion openapi_version: The OpenAPI version to use.
+    :param Version|str openapi_version: The OpenAPI version to use.
         Should be in the form '2.x' or '3.x.x' to comply with the OpenAPI standard.
     :param callable schema_name_resolver: Callable to generate the schema definition name.
         Receives the `Schema` class and returns the name to be used in refs within
@@ -47,13 +50,50 @@ class OpenAPIConverter(FieldConverterMixin):
         spec
     """
 
-    def __init__(self, openapi_version, schema_name_resolver, spec):
-        self.openapi_version = OpenAPIVersion(openapi_version)
+    def __init__(
+        self,
+        openapi_version: Version | str,
+        schema_name_resolver,
+        spec: APISpec,
+    ) -> None:
+        self.openapi_version = (
+            Version(openapi_version)
+            if isinstance(openapi_version, str)
+            else openapi_version
+        )
         self.schema_name_resolver = schema_name_resolver
         self.spec = spec
         self.init_attribute_functions()
+        self.init_parameter_attribute_functions()
         # Schema references
-        self.refs = {}
+        self.refs: dict = {}
+
+    def init_parameter_attribute_functions(self) -> None:
+        self.parameter_attribute_functions = [
+            self.field2required,
+            self.list2param,
+        ]
+
+    def add_parameter_attribute_function(self, func) -> None:
+        """Method to add a field parameter function to the list of field
+        parameter functions that will be called on a field to convert it to a
+        field parameter.
+
+        :param func func: the field parameter function to add
+            The attribute function will be bound to the
+            `OpenAPIConverter <apispec.ext.marshmallow.openapi.OpenAPIConverter>`
+            instance.
+            It will be called for each field in a schema with
+            `self <apispec.ext.marshmallow.openapi.OpenAPIConverter>` and a
+            `field <marshmallow.fields.Field>` instance
+            positional arguments and `ret <dict>` keyword argument.
+            May mutate `ret`.
+            User added field parameter functions will be called after all built-in
+            field parameter functions in the order they were added.
+        """
+        bound_func = func.__get__(self)
+        setattr(self, func.__name__, bound_func)
+        self.parameter_attribute_functions.append(bound_func)
 
     def resolve_nested_schema(self, schema):
         """Return the OpenAPI representation of a marshmallow Schema.
@@ -79,14 +119,14 @@ class OpenAPIConverter(FieldConverterMixin):
             if not name:
                 try:
                     json_schema = self.schema2jsonschema(schema_instance)
-                except RuntimeError:
+                except RuntimeError as exc:
                     raise APISpecError(
                         "Name resolver returned None for schema {schema} which is "
                         "part of a chain of circular referencing schemas. Please"
                         " ensure that the schema_name_resolver passed to"
                         " MarshmallowPlugin returns a string for all circular"
                         " referencing schemas.".format(schema=schema)
-                    )
+                    ) from exc
                 if getattr(schema, "many", False):
                     return {"type": "array", "items": json_schema}
                 return json_schema
@@ -95,7 +135,13 @@ class OpenAPIConverter(FieldConverterMixin):
         return self.get_ref_dict(schema_instance)
 
     def schema2parameters(
-        self, schema, *, location, name="body", required=False, description=None
+        self,
+        schema,
+        *,
+        location,
+        name: str = "body",
+        required: bool = False,
+        description: str | None = None,
     ):
         """Return an array of OpenAPI parameters given a given marshmallow
         :class:`Schema <marshmallow.Schema>`. If `location` is "body", then return an array
@@ -135,33 +181,59 @@ class OpenAPIConverter(FieldConverterMixin):
             for field_name, field_obj in fields.items()
         ]
 
-    def _field2parameter(self, field, *, name, location):
+    def _field2parameter(
+        self, field: marshmallow.fields.Field, *, name: str, location: str
+    ) -> dict:
         """Return an OpenAPI parameter as a `dict`, given a marshmallow
         :class:`Field <marshmallow.Field>`.
 
         https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.2.md#parameterObject
         """
-        ret = {"in": location, "name": name}
-
-        partial = getattr(field.parent, "partial", False)
-        ret["required"] = field.required and (
-            not partial or (is_collection(partial) and field.name not in partial)
-        )
+        ret: dict = {"in": location, "name": name}
 
         prop = self.field2property(field)
-        multiple = isinstance(field, marshmallow.fields.List)
-
         if self.openapi_version.major < 3:
-            if multiple:
-                ret["collectionFormat"] = "multi"
             ret.update(prop)
         else:
-            if multiple:
-                ret["explode"] = True
-                ret["style"] = "form"
-            if prop.get("description", None):
+            if "description" in prop:
                 ret["description"] = prop.pop("description")
             ret["schema"] = prop
+
+        for param_attr_func in self.parameter_attribute_functions:
+            ret.update(param_attr_func(field, ret=ret))
+
+        return ret
+
+    def field2required(
+        self, field: marshmallow.fields.Field, **kwargs: typing.Any
+    ) -> dict:
+        """Return the dictionary of OpenAPI parameter attributes for a required field.
+
+        :param Field field: A marshmallow field.
+        :rtype: dict
+        """
+        ret = {}
+        partial = getattr(field.parent, "partial", False)
+        ret["required"] = field.required and (
+            not partial
+            or (is_collection(partial) and field.name not in partial)  # type:ignore
+        )
+        return ret
+
+    def list2param(self, field: marshmallow.fields.Field, **kwargs: typing.Any) -> dict:
+        """Return a dictionary of parameter properties from
+        :class:`List <marshmallow.fields.List` fields.
+
+        :param Field field: A marshmallow field.
+        :rtype: dict
+        """
+        ret: dict = {}
+        if isinstance(field, marshmallow.fields.List):
+            if self.openapi_version.major < 3:
+                ret["collectionFormat"] = "multi"
+            else:
+                ret["explode"] = True
+                ret["style"] = "form"
         return ret
 
     def schema2jsonschema(self, schema):
@@ -177,9 +249,8 @@ class OpenAPIConverter(FieldConverterMixin):
         fields = get_fields(schema)
         Meta = getattr(schema, "Meta", None)
         partial = getattr(schema, "partial", None)
-        ordered = getattr(schema, "ordered", False)
 
-        jsonschema = self.fields2jsonschema(fields, partial=partial, ordered=ordered)
+        jsonschema = self.fields2jsonschema(fields, partial=partial)
 
         if hasattr(Meta, "title"):
             jsonschema["title"] = Meta.title
@@ -190,18 +261,17 @@ class OpenAPIConverter(FieldConverterMixin):
 
         return jsonschema
 
-    def fields2jsonschema(self, fields, *, ordered=False, partial=None):
+    def fields2jsonschema(self, fields, *, partial=None):
         """Return the JSON Schema Object given a mapping between field names and
         :class:`Field <marshmallow.Field>` objects.
 
         :param dict fields: A dictionary of field name field object pairs
-        :param bool ordered: Whether to preserve the order in which fields were declared
         :param bool|tuple partial: Whether to override a field's required flag.
             If `True` no fields will be set as required. If an iterable fields
             in the iterable will not be marked as required.
         :rtype: dict, a JSON Schema Object
         """
-        jsonschema = {"type": "object", "properties": OrderedDict() if ordered else {}}
+        jsonschema = {"type": "object", "properties": {}}
 
         for field_name, field_obj in fields.items():
             observed_field_name = field_obj.data_key or field_name
